@@ -1,8 +1,6 @@
-from __future__ import absolute_import, division, print_function
-
 import json as _json
-from six.moves.urllib.request import urlopen
-from six.moves.urllib.parse import urlencode, quote_plus
+from urllib.parse import urlencode, quote_plus
+from urllib.request import urlopen
 
 from .config import ServerConfig, _server_configs
 from .utils import _split_arg
@@ -10,7 +8,6 @@ from .utils import _split_arg
 __all__ = ['const', 'HttpJsonRequest', 'HttpJsonClient']
 
 
-# Constants for jsoc_info calls
 class JsocInfoConstants:
     """
     Constants for DRMS queries.
@@ -40,6 +37,7 @@ class JsocInfoConstants:
     archive
         = ``'*archive*'``
     """
+
     all = '**ALL**'
     none = '**NONE**'
     recdir = '*recdir*'
@@ -51,15 +49,18 @@ class JsocInfoConstants:
     online = '*online*'
     retain = '*retain*'
     archive = '*archive*'
+
+
 const = JsocInfoConstants()
 
 
-class HttpJsonRequest(object):
+class HttpJsonRequest:
     """
     Class for handling HTTP/JSON requests.
 
-    Use :class:`HttpJsonClient` to create an instance.
+    Use `HttpJsonClient` to create an instance.
     """
+
     def __init__(self, url, encoding):
         self._encoding = encoding
         self._http = urlopen(url)
@@ -67,7 +68,7 @@ class HttpJsonRequest(object):
         self._data = None
 
     def __repr__(self):
-        return '<HttpJsonRequest "%s">' % self.url
+        return f'<HttpJsonRequest: {self.url}>'
 
     @property
     def url(self):
@@ -86,25 +87,19 @@ class HttpJsonRequest(object):
         return self._data
 
 
-class HttpJsonClient(object):
+class HttpJsonClient:
     """
     HTTP/JSON communication with the DRMS server CGIs.
 
     Parameters
     ----------
-    server : string or drms.config.ServerConfig
+    server : str or drms.config.ServerConfig
         Registered server ID or ServerConfig instance.
         Defaults to JSOC.
     debug : bool
         Enable or disable debug mode (default is disabled).
-
-    Attributes
-    ----------
-    server : drms.config.ServerConfig
-        Remote server configuration.
-    debug : bool
-        Enable/disable debug output.
     """
+
     def __init__(self, server='jsoc', debug=False):
         if isinstance(server, ServerConfig):
             self._server = server
@@ -113,7 +108,7 @@ class HttpJsonClient(object):
         self.debug = debug
 
     def __repr__(self):
-        return '<HttpJsonClient "%s">' % self._server.name
+        return f'<HttpJsonClient: {self._server.name}>'
 
     def _json_request(self, url):
         if self.debug:
@@ -138,14 +133,14 @@ class HttpJsonClient(object):
 
         Parameters
         ----------
-        ds_filter : string
+        ds_filter : str
             Name filter regexp.
 
         Returns
         -------
         result : dict
         """
-        query = '?' if ds_filter is not None else ''
+        query = '?' if ds_filter is not None else ""
         if ds_filter is not None:
             query += urlencode({'filter': ds_filter})
         req = self._json_request(self._server.url_show_series + query)
@@ -162,7 +157,7 @@ class HttpJsonClient(object):
 
         Parameters
         ----------
-        ds_filter : string
+        ds_filter : str
             Name filter regexp.
         info : bool
             If False (default), the result only contains series names.
@@ -178,7 +173,7 @@ class HttpJsonClient(object):
             query_args['filter'] = ds_filter
         if info:
             query_args['info'] = '1'
-        query = '?' + urlencode(query_args)
+        query = f'?{urlencode(query_args)}'
         req = self._json_request(self._server.url_show_series_wrapper + query)
         return req.data
 
@@ -188,7 +183,7 @@ class HttpJsonClient(object):
 
         Parameters
         ----------
-        ds : string
+        ds : str
             Name of the data series.
 
         Returns
@@ -196,7 +191,7 @@ class HttpJsonClient(object):
         result : dict
             Dictionary containing information about the data series.
         """
-        query = '?' + urlencode({'op': 'series_struct', 'ds': ds})
+        query = f'?{urlencode({"op": "series_struct", "ds": ds})}'
         req = self._json_request(self._server.url_jsoc_info + query)
         return req.data
 
@@ -206,7 +201,7 @@ class HttpJsonClient(object):
 
         Parameters
         ----------
-        ds : string
+        ds : str
             Record set query (only one series).
 
         Returns
@@ -214,24 +209,23 @@ class HttpJsonClient(object):
         result : dict
             Dictionary containg 'count', 'status' and 'runtime'.
         """
-        query = '?' + urlencode({'op': 'rs_summary', 'ds': ds})
+        query = f'?{urlencode({"op": "rs_summary", "ds": ds})}'
         req = self._json_request(self._server.url_jsoc_info + query)
         return req.data
 
-    def rs_list(self, ds, key=None, seg=None, link=None, recinfo=False,
-                n=None, uid=None):
+    def rs_list(self, ds, key=None, seg=None, link=None, recinfo=False, n=None, uid=None):
         """
         Get detailed information about a record set.
 
         Parameters
         ----------
-        ds : string
+        ds : str
             Record set query.
-        key : string, list or None
+        key : str, list or None
             List of requested keywords, optional.
-        seg : string, list or None
+        seg : str, list or None
             List of requested segments, optional.
-        link : string or None
+        link : str or None
             List of requested Links, optional.
         recinfo : bool
             Request record info for each record in the record set.
@@ -240,7 +234,7 @@ class HttpJsonClient(object):
             of the record set are returned, for negative values the
             last abs(n) records. If set to None (default), no limit is
             applied.
-        uid : string or None
+        uid : str or None
             Session ID used when calling rs_list CGI, optional.
 
         Returns
@@ -260,69 +254,69 @@ class HttpJsonClient(object):
         if recinfo:
             d['R'] = '1'
         if n is not None:
-            d['n'] = '%d' % int(n)
+            d['n'] = f'{int(int(n))}'
         if uid is not None:
             d['userhandle'] = uid
-        query = '?' + urlencode(d)
+        query = f'?{urlencode(d)}'
         req = self._json_request(self._server.url_jsoc_info + query)
         return req.data
 
     def check_address(self, email):
         """
-        Check if an email address is registered for export data
-        requests.
+        Check if an email address is registered for export data requests.
 
         Parameters
         ----------
-        email : string
+        email : str
             Email address to be verified.
 
         Returns
         -------
         result : dict
-            Dictionary containing 'status' and 'msg'. Some status
-            codes are:
-                2: Email address is valid and registered
-                4: Email address has neither been validated nor
-                   registered
-               -2: Not a valid email address
+            Dictionary containing 'status' and 'msg'.
+
+            Some status codes are:
+            - 2: Email address is valid and registered
+            - 4: Email address has neither been validated nor registered
+            - -2: Not a valid email address
         """
-        query = '?' + urlencode({
-            'address': quote_plus(email), 'checkonly': '1'})
+        query = '?' + urlencode({'address': quote_plus(email), 'checkonly': '1'})
         req = self._json_request(self._server.url_check_address + query)
         return req.data
 
-    def exp_request(self, ds, notify, method='url_quick', protocol='as-is',
-                    protocol_args=None, filenamefmt=None, n=None,
-                    requestor=None):
+    def exp_request(self, *args, **kwargs):
         """
         Request data export.
 
         Parameters
         ----------
-        ds : string
+        ds : str
             Data export record set query.
-        notify : string
+        notify : str
             Registered email address.
-        method : string
+        method : str
             Export method. Supported methods are: 'url_quick', 'url',
             'url-tar', 'ftp' and 'ftp-tar'. Default is 'url_quick'.
-        protocol : string
+        protocol : str
             Export protocol. Supported protocols are: 'as-is', 'fits',
             'jpg', 'mpg' and 'mp4'. Default is 'as-is'.
         protocol_args : dict or None
             Extra protocol arguments for protocols 'jpg', 'mpg' and
             'mp4'. Valid arguments are: 'ct', 'scaling', 'min', 'max'
             and 'size'.
-        filenamefmt : string, None
+        filenamefmt : str, None
             Custom filename format string for exported files. This is
             ignored for 'url_quick'/'as-is' data exports.
+        process : `dict`, None
+            Dictionary of processing commands. Each entry is also a `dict`
+            containing all of the applicable options for that processing
+            command.
         n : int or None
             Limits the number of records requested. For positive
             values, the first n records of the record set are returned,
             for negative values the last abs(n) records. If set to None
             (default), no limit is applied.
-        requestor : string, None or False
+        requestor : str, None or bool
             Export user ID. Default is None, in which case the user
             name is determined from the email address. If set to False,
             the requestor argument will be omitted in the export
@@ -334,20 +328,39 @@ class HttpJsonClient(object):
             Dictionary containing the server response to the export
             request.
         """
+        req = self._json_request(self._exp_request_url(*args, **kwargs))
+        return req.data
+
+    def _exp_request_url(
+        self,
+        ds,
+        notify,
+        method='url_quick',
+        protocol='as-is',
+        protocol_args=None,
+        filenamefmt=None,
+        n=None,
+        process=None,
+        requestor=None,
+    ):
         method = method.lower()
         method_list = ['url_quick', 'url', 'url-tar', 'ftp', 'ftp-tar']
         if method not in method_list:
             raise ValueError(
-                "Method '%s' is not supported, valid methods are: %s" %
-                (method, ', '.join("'%s'" % s for s in method_list)))
+                'Method {} is not supported, valid methods are: {}'.format(
+                    method, ', '.join(str(s) for s in method_list)
+                )
+            )
 
         protocol = protocol.lower()
         img_protocol_list = ['jpg', 'mpg', 'mp4']
         protocol_list = ['as-is', 'fits'] + img_protocol_list
         if protocol not in protocol_list:
             raise ValueError(
-                "Protocol '%s' is not supported, valid protocols are: %s" %
-                (protocol, ', '.join("'%s'" % s for s in protocol_list)))
+                'Protocol {} is not supported, valid protocols are: {}'.format(
+                    protocol, ', '.join(str(s) for s in protocol_list)
+                )
+            )
 
         # method "url_quick" is meant to be used with "as-is", change method
         # to "url" if protocol is not "as-is"
@@ -367,26 +380,48 @@ class HttpJsonClient(object):
                     elif k in ['min', 'max']:
                         d[k] = float(v)
                     else:
-                        raise ValueError("Unknown protocol argument: '%s'" % k)
+                        raise ValueError(f'Unknown protocol argument: {k}')
             protocol += ',CT={ct},scaling={scaling},size={size}'.format(**d)
             if 'min' in d:
-                protocol += ',min=%g' % d['min']
+                protocol += f',min={d["min"]:g}'
             if 'max' in d:
-                protocol += ',max=%g' % d['max']
+                protocol += f',max={d["max"]:g}'
         else:
             if protocol_args is not None:
-                raise ValueError(
-                    "protocol_args not supported for protocol '%s'" % protocol)
+                raise ValueError(f'protocol_args not supported for protocol {protocol}')
 
-        d = {'op': 'exp_request', 'format': 'json', 'ds': ds,
-             'notify': notify, 'method': method, 'protocol': protocol}
+        d = {
+            'op': 'exp_request',
+            'format': 'json',
+            'ds': ds,
+            'notify': notify,
+            'method': method,
+            'protocol': protocol,
+        }
 
         if filenamefmt is not None:
             d['filenamefmt'] = filenamefmt
 
-        if n is not None:
-            n = int(n)
-            d['process=n'] = '%d' % n
+        n = int(n) if n is not None else 0
+        d['process=n'] = f'{n}'
+        if process is not None:
+            allowed_processes = [
+                'im_patch',
+                'resize',
+                'rebin',
+                'aia_scale_aialev1',
+                'aia_scale_orig',
+                'aia_scale_other',
+                'Maproj',
+                'HmiB2ptr',
+            ]
+            process_strings = {}
+            for p, opts in process.items():
+                if p not in allowed_processes:
+                    raise ValueError(f'{p} is not one of the allowed processing options: {allowed_processes}')
+                process_strings[p] = ','.join([f'{k}={v}' for k, v in opts.items()])
+            processes = '|'.join([f'{k},{v}' for k, v in process_strings.items()])
+            d['process=n'] = f'{d["process=n"]}|{processes}'
 
         if requestor is None:
             d['requestor'] = notify.split('@')[0]
@@ -394,8 +429,7 @@ class HttpJsonClient(object):
             d['requestor'] = requestor
 
         query = '?' + urlencode(d)
-        req = self._json_request(self._server.url_jsoc_fetch + query)
-        return req.data
+        return self._server.url_jsoc_fetch + query
 
     def exp_status(self, requestid):
         """
@@ -403,7 +437,7 @@ class HttpJsonClient(object):
 
         Parameters
         ----------
-        requestid : string
+        requestid : str
             Request identifier returned by exp_request.
 
         Returns
@@ -411,6 +445,6 @@ class HttpJsonClient(object):
         result : dict
             Dictionary containing the export request status.
         """
-        query = '?' + urlencode({'op': 'exp_status', 'requestid': requestid})
+        query = f'?{urlencode({"op": "exp_status", "requestid": requestid})}'
         req = self._json_request(self._server.url_jsoc_fetch + query)
         return req.data
